import datetime

def time_in_range(start, end, x):
    if start <= end:
        return start <= x <= end
    else:
        return start <= x or x <= end

start = datetime.time(23, 0, 0)
end = datetime.time(4, 0, 0)

print(time_in_range(start, end, datetime.datetime.now().time()))


from pathlib import Path
from winreg import OpenKey, HKEY_CURRENT_USER, QueryValueEx

import dropbox


class DropboxHandler:

    def __init__(self, apikey, toaster):
        self.dbx = dropbox.Dropbox(apikey, timeout=30)
        self.CHUNK_SIZE = 4 * 1024 * 1024
        self.DEFAULT_REMOTE_PATH = "/"
        with OpenKey(
            HKEY_CURRENT_USER,
            'SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders'
        ) as key:
            self.DEFAULT_LOCAL_PATH = QueryValueEx(
                key, '{374DE290-123F-4565-9164-39C4925E467B}'
            )[0] + "\\"
        self.toaster = toaster

    # Dropbox download
    def download_file(self, remote_path, local_path):
        if not remote_path:
            raise Exception("Download source is missing")
        filename = remote_path.split("/")[-1]
        print(f"Getting {filename} from Dropbox...")
        dest_file = local_path or self.DEFAULT_LOCAL_PATH + filename

        try:
            self.dbx.files_download_to_file(dest_file, remote_path)
        except dropbox.exceptions.ApiError:
            print("Error: file cannot be found!")
            return
        print("File has finished downloading to", dest_file)

        print("Removing from cloud...")
        try:
            self.dbx.files_delete_v2(remote_path)
        except dropbox.exceptions.ApiError:
            print("Error: file cannot be deleted!")
            return
        print("Done")

    def upload_file(self, local_path, remote_path):
        """
        Uploads a file to Dropbox and returns the temporary (24 hours) link
        """

        if not local_path:
            raise Exception("Upload source is missing")

        # Process file
        file_name = local_path.split("\\")[-1]
        dest_path = remote_path or self.DEFAULT_REMOTE_PATH + file_name

        read_file = open(local_path, 'rb')
        file_size = Path(local_path).stat().st_size
        file_size_megs = "{0:.2f} KB".format(file_size/1024)

        # Upload file
        print(f"Uploading {file_name} ({file_size_megs})")
        if file_size <= self.CHUNK_SIZE:
            self.dbx.files_upload(
                read_file.read(),
                dest_path,
                mute=True
            )
            print("Done")
        else:
            upload_session_start_result = self.dbx.files_upload_session_start(
                read_file.read(self.CHUNK_SIZE)
            )
            cursor = dropbox.files.UploadSessionCursor(
                session_id=upload_session_start_result.session_id,
                offset=read_file.tell()
            )
            commit = dropbox.files.CommitInfo(path=dest_path)

            self.toaster.show_toast(
                title="Sending file",
                msg=f"{file_name} ({file_size_megs})",
                duration=0,
                threaded=True
            )

            # Upload in chunks and show status bar
            toolbar_width = 30

            while read_file.tell() < file_size:
                if ((file_size - read_file.tell()) <= self.CHUNK_SIZE):
                    self.dbx.files_upload_session_finish(
                        read_file.read(self.CHUNK_SIZE),
                        cursor, commit
                    )
                else:
                    self.dbx.files_upload_session_append(
                        read_file.read(self.CHUNK_SIZE),
                        cursor.session_id, cursor.offset
                    )
                    cursor.offset = read_file.tell()
                    progress = cursor.offset/file_size
                    bars = int(toolbar_width*progress)
                    print(
                        f"{'█' * bars}{'░' * (toolbar_width - bars)} "
                        f"{int(progress*100)}%",
                        end="\r"
                    )
            print(f"Done{' ' * (toolbar_width)}", end="\r")

            self.toaster.show_toast(
                title="File sent",
                msg=f"{file_name} ({file_size_megs})",
                duration=0,
                threaded=True
            )

        return self.dbx.files_get_temporary_link(dest_path)



from base64 import decodebytes
from binascii import hexlify
import re
import threading

import paramiko


class Server(paramiko.ServerInterface):

    def __init__(self, pub_key_file, priv_key_file, req_proc, username):
        self.event = threading.Event()
        self.host_key = paramiko.RSAKey(filename=priv_key_file)
        self.req_proc = req_proc
        self.username = username

        f = open(pub_key_file, 'r')
        pub_key_raw = b''
        for line in f.read().split('\n'):
            if not re.match(r'----|Comment:|^$', line):
                pub_key_raw += bytes(line, 'utf-8')
        self.pub_key = paramiko.RSAKey(
            data=decodebytes(pub_key_raw)
        )

    def check_channel_request(self, kind, chanid):
        if kind == 'session':
            return paramiko.OPEN_SUCCEEDED

    def check_auth_publickey(self, username, key):
        print(
            f"Auth attempt:\n"
            f"- Username: {username}\n"
            f"- Key fingerprint:",
            hexlify(key.get_fingerprint()).decode('utf-8')
        )
        if (username == "ABS") and (key == self.pub_key):
            print("✔ Auth successful")
            return paramiko.AUTH_SUCCESSFUL
        print("✗ Auth failed")
        return paramiko.AUTH_FAILED

    def get_allowed_auths(self, username):
        return 'publickey'

    def check_channel_exec_request(self, channel, command):
        request_processor_thread = threading.Thread(
            target=self.req_proc.process_external_request(command)
        )
        request_processor_thread.start()
        self.event.set()
        return True



import configparser
import logging
import os
import socket
import time
import subprocess
import sys
from shutil import copyfile

import paramiko
from win10toast import ToastNotifier

from spesdebris.dbxhandler import DropboxHandler
from spesdebris.fcmhandler import FCMHandler
from spesdebris.extrq import RequestProcessor
from spesdebris.server import Server


def cleanup():
    sys.exit(0)


def open_config(args=None):
    subprocess.run("notepad " + config_location)


# Load configuration or perform first time setup
config_location = os.getenv('LOCALAPPDATA') + '\\spesdebris\\settings.ini'
if not os.path.isfile(config_location):
    print("The settings file was not found, please edit it.")
    try:
        os.mkdir(os.path.dirname(config_location))
    except FileExistsError:
        pass
    copyfile(
        os.path.dirname(__file__) + "\\settings-sample.ini",
        config_location
    )
    open_config()
    if not os.path.isfile(config_location):
        print("The settings file could not be created, quitting.")
        cleanup()

config = configparser.ConfigParser()
config.read(config_location)

port = config.getint('Server', 'Port')
toaster = ToastNotifier()
dbh = DropboxHandler(
    config.get('APIkeys', 'Dropbox'),
    toaster
)
fcmh = FCMHandler(
    config.get('Client', 'Account'),
    config.get('APIkeys', 'Automate')
)
reqproc = RequestProcessor(
    dbh, fcmh, toaster
)

logging.basicConfig()
logger = logging.getLogger()


def listener():
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.bind(('', port))

    sock.listen(100)
    client, addr = sock.accept()
    print(
        f"[{time.strftime('%H:%M:%S', time.localtime(time.time()))}"
        f"] Incoming connection from {addr}"
    )

    t = paramiko.Transport(client)
    t.set_gss_host(socket.getfqdn(""))
    t.load_server_moduli()
    server = Server(
        config.get('Server', 'PublicKeyFile'),
        config.get('Server', 'PrivateKeyFile'),
        reqproc,
        config.get('Server', 'Username')
    )
    t.add_server_key(server.host_key)
    t.start_server(server=server)

    # Wait 30 seconds for a command
    server.event.wait(30)
    t.close()
    print("Connection closed\n")


def daemon_mode(args):
    print(
        "spesdebris daemon started\n"
        f"Starting SSH server on port {port}\n"
    )
    while True:
        try:
            listener()
        except KeyboardInterrupt:
            print("spesdebris daemon stopped")
            cleanup()
        except Exception as exc:
            logger.error(exc)


def upload_to_phone(args):
    if not args.source_file:
        raise Exception("Error: the source file is not specified")

    file_name = args.source_file.split("\\")[-1]
    upload_result = dbh.upload_file(
        local_path=args.source_file,
        remote_path=None
    )

    fcmh.send_message(
        device=config.get('Client', 'Device'),
        payload={
            "command": "Get file",
            "from": upload_result.link,
            "to": args.destination or config.get(
                'Client', 'DefaultDownloadFolder') + file_name,
            "filename": file_name
        }
    )

    cleanup()


def send_fcm_message(args):
    fcmh.send_message(
        device=config.get('Client', 'Device'),
        payload={
            "command": "Show message",
            "message": args.message
        }
    )

    cleanup()


import argparse
from sys import argv

import spesdebris.__init__ as main

# Parse command line arguments if running standalone
if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="""
        spesdebris: PC automation and cross-device communication suite
        """
    )
    subparsers = parser.add_subparsers(help='sub-command help')

    parser_upload = subparsers.add_parser(
        "upload",
        help='Upload the specified file to your Android device'
    )
    parser_upload.add_argument('source_file', type=str)
    parser_upload.add_argument('--destination','-d', type=str)
    parser_upload.set_defaults(func=main.upload_to_phone)

    parser_daemon = subparsers.add_parser(
        "daemon",
        help='Run spesdebris as a background process'
    )
    parser_daemon.set_defaults(func=main.daemon_mode)

    parser_fcm_send = subparsers.add_parser(
        "fcm_send",
        help="Send FCM message"
    )
    parser_fcm_send.add_argument('message', type=str)
    parser_fcm_send.set_defaults(func=main.send_fcm_message)

    parser_configure = subparsers.add_parser(
        "configure",
        help="Open spesdebris configuration"
    )
    parser_configure.set_defaults(func=main.open_config)

    args = parser.parse_args()

    if len(argv) > 1:
        args.func(args)
    else:
        parser.parse_args("--help".split())



from base64 import decodebytes
import json
from subprocess import call, CalledProcessError
from time import sleep
import webbrowser

import pyperclip
# from win32gui import Shell_NotifyIcon, NIM_DELETE


class RequestProcessor():
    """
    This class processes all the requests received from
    your Android device.
    """

    def __init__(self, DropboxHandler, FCMHandler, toaster):
        self.dbh = DropboxHandler
        self.fcmh = FCMHandler
        self.toaster = toaster

    def process_external_request(self, raw_payload):
        """
        This method decodes the incoming base64 and JSON encoded payload
        and executes the requested command
        """

        commands = {
            'disnot': self.dismiss_notification,
            'echo': self.echo,
            'getfile': self.get_file,
            'mirnot': self.mirror_notification,
            'openlink': self.open_link,
            'setclip': self.set_clipboard,
            'setpwr': self.set_power,
            'setvol': self.set_volume,
        }

        decoded_payload = decodebytes(raw_payload).decode('utf-8')
        print(">", decoded_payload)
        request = json.loads(decoded_payload)

        try:
            commands[request["command"]](request)
        except KeyError:
            print(f"Error: {request['command']} is not a valid command")
        except CalledProcessError as e:
            print(f"Info: Command '{e.cmd}' "
                  f"returned with error code {e.returncode}")

    def echo(self, cmdin):
        txt = cmdin["text"]
        print(txt)

    def set_power(self, cmdin):
        power_state = cmdin["powerstate"]
        psc = {
            'sleep': 'rundll32.exe powrprof.dll,SetSuspendState 0,1,0',
            'off': 'shutdown /s',
            'reboot': 'shutdown /r'
        }
        call(psc[power_state], shell=True)

    def set_volume(self, cmdin):
        s = cmdin["volume"]
        if s == "mute" or s == "unmute":
            call(
                f"nircmdc mutesysvolume {'1' if s == 'mute' else '0'}",
                shell=True
            )

    def set_clipboard(self, cmdin):
        s = cmdin["text"]
        pyperclip.copy(
            s.replace("\n", "\r\n")
        )

    def mirror_notification(self, cmdin):
        self.toaster.show_toast(
            title=cmdin["title"],
            msg=cmdin["message"],
            duration=10,
            threaded=True
        )

    def dismiss_notification(self, cmdin):
        # TODO
        print("destroy", cmdin["id"])
        # Shell_NotifyIcon(NIM_DELETE, (self.toaster.hwnd, 0))

    def get_file(self, cmdin):
        # Dropbox needs a little time to register the new file ¯\_(ツ)_/¯
        sleep(3)
        self.dbh.download_file(
            remote_path=cmdin["pathFrom"],
            local_path=None
        )

    def open_link(self, cmdin):
        webbrowser.open(url=cmdin["link"])



import json
import requests
import base64


class FCMHandler:

    def __init__(self, account, secret):
        self.account = account
        self.secret = secret

    def send_message(self, device, payload):
        print("Sending cloud message")

        json_request = {
            "secret": self.secret,
            "to": self.account,
            "device": device,
            "payload": base64.urlsafe_b64encode(
                json.dumps(payload).encode()
            )
        }
        response = requests.post(
            'http://llamalab.com/automate/cloud/message', data=json_request)
        print("Message sent successfully" if response.status_code ==
              200 else f"Remote response: {response.status_code}")



import setuptools

with open("readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="spesdebris",
    version="0.0.3",
    author="Benedek Solt Antók",
    author_email="abs@abs.ezw.me",
    description="Python-based desktop automation"
    "and cross-device communication suite",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ABS96/spesdebris",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: Microsoft :: Windows :: Windows 10"
    ],
)
